@@ -1,8 +1,3 @@
-#!/usr/bin/env python3
-# TODO: check for error with using a proxy to get proxies
-# TODO: `rg` to find any other todos before release
-# TODO: strongly type parameters
-
 import requests
 
 from datetime import datetime as dt
@@ -13,6 +8,7 @@ from urllib.parse import urlencode
 
 from pubproxpy._singleton import Singleton
 from pubproxpy.errors import ProxyError, API_ERROR_MAP
+from pubproxpy.types import Level, Protocol
 
 
 class _FetcherShared(metaclass=Singleton):
@@ -30,9 +26,6 @@ class _FetcherShared(metaclass=Singleton):
         self.__init__()
 
 
-# TODO: set up tests for things
-# TODO: move all the constants for ProxyFetcher outside of the class?
-#       to constants file maybe?
 class ProxyFetcher:
     """Class used to fetch proxies from the pubproxy API matching the provided
     parameters
@@ -58,12 +51,6 @@ class ProxyFetcher:
         "user_agent",
     )
 
-    # Parameters that have explicit options
-    _PARAM_OPTS = {
-        "level": ("anonymous", "elite"),
-        "protocol": ("http", "socks4", "socks5"),
-    }
-
     # Parameters that are bounded
     _PARAM_BOUNDS = {"last_checked": (1, 1000), "time_to_connect": (1, 60)}
 
@@ -106,29 +93,37 @@ class ProxyFetcher:
         """
 
         # `countries` and `not_countries` are mutually exclusive
-        assert "countries" not in params or "not_countries" not in params, (
-            "incompatible parameters, `countries` and `not_countries` are"
-            " mutually exclusive"
-        )
+        if "countries" in params and "not_countries" in params:
+            raise ValueError(
+                "incompatible parameters, `countries` and `not_countries` are"
+                " mutually exclusive"
+            )
+
+        # Check that protocol and level are the correct type
+        for key, enum_type in (("protocol", Protocol), ("level", Level)):
+            if key in params:
+                val = params[key]
+                if not isinstance(val, enum_type):
+                    raise ValueError(
+                        f"{key} should be of type `{enum_type}` not "
+                        f" `{type(val)}`"
+                    )
 
         # Verify all params are valid, and satisfy the valid bounds or options
         for param, val in params.items():
-            assert param in self._PARAMS, (
-                f'invalid parameter "{param}" valid parameters are'
-                f" {[p for p in self._PARAMS]}"
-            )
-
-            if param in self._PARAM_OPTS:
-                opts = self._PARAM_OPTS[param]
-                assert (
-                    val in opts
-                ), f'invalid value "{val}" for "{param}" options are {opts}'
+            if param not in self._PARAMS:
+                raise ValueError(
+                    f'unrecognized parameter "{param}" valid parameters are'
+                    f" {[p for p in self._PARAMS]}"
+                )
 
             if param in self._PARAM_BOUNDS:
                 low, high = self._PARAM_BOUNDS[param]
-                assert (
-                    low <= val <= high
-                ), f'value "{val}" for "{param}" out of bounds ({low}, {high})'
+                if val < low or val > high:
+                    raise ValueError(
+                        f'value "{val}" for "{param}" out of bounds'
+                        f" ({low} to {high})"
+                    )
 
     def _rename_params(self, params):
         """Method to rename some params from the API's method to pubproxy's
@@ -171,19 +166,19 @@ class ProxyFetcher:
             if isinstance(params["not_country"], (list, tuple)):
                 params["not_country"] = ",".join(params["not_country"])
 
+        # Get value from enums
+        for key in ("level", "type"):
+            if key in params:
+                params[key] = params[key].value
+
         return params
 
     def drain(self):
         """Returns any proxies remaining in the current list
         """
-        return self.get_proxies(len(self._proxies))
+        return self.get(len(self._proxies))
 
-    def get_proxy(self):
-        """Attempts to get a single proxy matching the specified params
-        """
-        return self.get_proxies(1)[0]
-
-    def get_proxies(self, amount):
+    def get(self, amount=1):
         """Attempts to get `amount` proxies matching the specified params
         """
         # Remove any blacklisted proxies from the internal list
@@ -224,6 +219,8 @@ class ProxyFetcher:
 
         # Query the api
         resp = requests.get(self._query)
+        # And ensure the response is ok
+        resp.raise_for_status()
 
         try:
             data = json.loads(resp.text)["data"]
